#!/usr/bin/perl
# $Id: dynamicsign.pl,v 1.6.2.1 2017/02/22 11:34:18 rjs Exp $
#
# PDFlib PLOP DS:
# generate a dynamic visualization document in memory with PDFlib,
# and digitally sign a second document, using the dynamic document
# for signature visualization.

use PDFlib::PLOP 5.2;
use PDFlib::PDFlib 9.0;

my $doc;
my $contents;
my @months = ( "January", "February", "March", "April", "May", "June",
            "July", "August", "September", "October", "November", "December");

# This is where input files live. Adjust as necessary.
my $searchpath = "../data";

# Basic signature options
$sign_opts =
    "ltv=try " .         # use ltv=full to ensure LTV status
    # when using PKCS11 use
    # "engine=pkcs#11 digitalid={filename=cryptoki.dll} "
    "engine=builtin digitalid={filename=demo_signer_rsa_2048.p12} password={demo} " ;

if ($#ARGV != 1 ) {
    die("usage: dynamicsign.pl <infile> <outfile> \n");
}


# --------- Step 1: create visualization document in memory with PDFlib --------
my $p = new PDFlib::PDFlib;

eval {
    # create a new PDF document in memory

    $p->begin_document("", "");

    $p->set_info("Creator", "dynamicsign.pl");
    $p->set_info("Author", "Rainer Plckl");
    $p->set_info("Title", "Dynamic signature visualization document");

    $p->begin_page_ext(300, 50, "");
    # Prepare dynamic text for the signature visualization 
    my $timebuf = sprintf("Digitally signed %s %d, %d at %02d:%02d:%02d",
	     $months[(localtime)[4]], (localtime)[3], (localtime)[5]+1900,
	    (localtime)[2], (localtime)[1],(localtime)[0]);

    my $textoptlist =sprintf( "fontsize=10 fontname=Helvetica " . 
	    "encoding=unicode textformat=utf8 fillcolor=red");
    $p->fit_textline($timebuf, 10, 10, $textoptlist);
    $p->end_page_ext("");

    $p->end_document("");

    # fetch the buffer containing the PDF document
    $contents = $p->get_buffer();
};

if ($@) {
    die("$0: PDFlib Exception occurred in dynamicsign sample:\n$@");
}

# ------------------ Step 2: sign the document -----------------
    
# create a new plop object
my $plop = new PDFlib::PLOP;

eval {
    $plop->set_option("searchpath={${searchpath}}");

    # Create a virtual PVF file from the document data in memory
    $plop->create_pvf("/pvf/visdoc.pdf", $contents, "");

    if (($doc = $plop->open_document($ARGV[0], "")) == -1)
    {
	die("Error: ", $plop->get_apiname(), ": ", $plop->get_errmsg(), "\n");
    }
    
    # open visualization document from PVF file
    if (($visdoc = $plop->open_document("/pvf/visdoc.pdf", "")) == -1)
    {
	die("Error: ", $plop->get_apiname(), ": ", $plop->get_errmsg(), "\n");
    }
    # Add field option for visualization to base signature options
    $optlist = sprintf("%s field={visdoc=%d rect={10 10 adapt adapt}} ",
		    $sign_opts, $visdoc);
    if ($plop->prepare_signature($optlist) == -1)
    {
        die("Error: ", $plop->get_apiname(), ": ", $plop->get_errmsg(), "\n");
    }

    # sign the output file
    my $optlist = sprintf("input=%d", $doc);
    if ($plop->create_document($ARGV[1], $optlist) == -1)
    {
	die("Error: ", $plop->get_apiname(), ": ", $plop->get_errmsg(), "\n");
    }

    # close the input document
    $plop->close_document($doc, "");

    # close the visualization document
    $plop->close_document($visdoc, "");

    # delete PVF file containing input document
    if ($plop->delete_pvf("/pvf/visdoc.pdf") == -1)
    {
	die("Error: ", $plop->get_apiname(), ": ", $plop->get_errmsg(), "\n");
    }
};

if ($@) {
    die("$0: PLOP Exception occurred:\n$@");
}
