#!/usr/bin/perl
# $Id: insertxmp.pl,v 1.10.2.1 2017/02/22 11:34:18 rjs Exp $
#
# PDFlib PLOP: PDF Linearization, Optimization, Protection
# mini sample for inserting XMP metadata in PDF documents
#
#

use PDFlib::PLOP 5.2;

# Directory where files will be searched
my $searchpath = "../data";
# input document
my $in_filename  = "PLOP-datasheet.pdf";
# output document
my $out_filename = "PLOP-datasheet-xmp.pdf";

# Option list for the generated output document.
# Some alternatives for the option list using the supplied XMP samples:
#
# Insert custom metadata for the GWG (Ghent Workgroup)
# Ad Ticket scheme (see http://www.gwg.org/Jobtickets.phtml):
# "metadata={filename=gwg_ad_ticket.xmp validate=xmp2004}"
#
# Insert custom metadata based on Acme's enterprise schema:
# "metadata={filename=acme.xmp}"
#
# Insert a custom "engineering" schema for scanned documents:
# This works best with a PDF/A-1 input document (e.g. mini_pdfa1.pdf),
# and requires XMP input which conforms to PDF/A-1:
# "metadata={filename=engineering.xmp}"
#
# Insert a PDF/A extension schema after validating the XMP for PDF/A-1.
# This works best with a PDF/A-1 input document (e.g. mini_pdfa1.pdf),
# and requires XMP input which conforms to PDF/A-1:
# "metadata={filename=machine_pdfa1.xmp validate=pdfa1}"
#

$base_opts = "metadata={filename=gwg_ad_ticket.xmp} ";

# create a new PLOP object
my $plop = new PDFlib::PLOP;

eval {
    my $doc;
    my $optlist;
    $optlist = "searchpath={$searchpath}";
    $plop->set_option($optlist);

    # open input file
    if (($doc = $plop->open_document($in_filename, "")) == -1)
    {
	die("Error: %s", $plop->get_errmsg());
    };

    # create the output file and add XMP metadata
    $optlist = sprintf("%s input=%d", $base_opts, $doc);
    if ($plop->create_document($out_filename, $optlist) == -1)
    {
	die("Error: %s", $plop->get_errmsg());
    }

    # close input files
    $plop->close_document($doc, "");
};
if ($@) {
    die("$0: PLOP Exception occurred:\n$@");
}
