#!/usr/bin/perl
# $Id: sign.pl,v 1.26.2.1 2017/02/22 11:34:18 rjs Exp $
#
# PDFlib PLOP DS:
# Demonstrate digital signature features with optional signature visualization
#

use PDFlib::PLOP 5.2;

my $doc;
# create a new PLOP object
my $plop = new PDFlib::PLOP;

# Basic signature options 
$sign_opts =
    "ltv=try " .         # use ltv=full to ensure LTV status 
    # when using PKCS11 use
    # "engine=pkcs#11 digitalid={filename=cryptoki.dll} "
    "engine=builtin digitalid={filename=demo_signer_rsa_2048.p12} password={demo} " ;

# This is where input files live. Adjust as necessary.
my $searchpath = "../data";

#  Set to 1 to enable signature visualization
my $visualize = 0;
my $visfile = "signing_man_pdfa1a.pdf";

if ($#ARGV != 1 ) {
    die("usage: sign <infile> <outfile> \n");
}

eval {
    my $optlist;
    $plop->set_option("searchpath={${searchpath}}");

    if (($doc = $plop->open_document($ARGV[0], "")) == -1)
    {
	die("Error: ", $plop->get_apiname(), ": ", $plop->get_errmsg(), "\n");
    }
    # Basic signature options 
    $optlist = $sign_opts;
    if ($visualize){

	if (($visdoc = $plop->open_document($visfile, "")) == -1)
	{
	    die("Error: ", $plop->get_apiname(), ": ", $plop->get_errmsg(), "\n");
	}
	$optlist .= sprintf("field={visdoc=%d rect={50 50 adapt adapt}} ", $visdoc);
    }

    if ($plop->prepare_signature($optlist) == -1)
    {
	die("Error: ", $plop->get_apiname(), ": ", $plop->get_errmsg(), "\n");
    }

    $optlist = sprintf("input=%d", $doc);
    if ($plop->create_document($ARGV[1], $optlist) == -1)
    {
	die("Error: ", $plop->get_apiname(), ": ", $plop->get_errmsg(), "\n");
    }

    $plop->close_document($doc, "");
    if ($visualize){
	$plop->close_document($visdoc, "");
    }
};

if ($@) {
    die("$0: PLOP Exception occurred:\n$@");
}
